/*
 * v4l_tools    A small set of tools for video4linux
 * Copyright (C) 2002, 2003 Raphael Assenat
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Library General Public
 * License as published by the Free Software Foundation; either
 * version 2 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warrenty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Library General Public License for more details.
 *
 * You should have received a copy of the GNU Library General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307 USA
 *
 * Raphael Assenat
 * raph@raphnet.net
 * http://www.raphnet.net
 *
 * */
#include <stdio.h>
#include <unistd.h>
#include <stdlib.h>
#include <sys/stat.h>
#include <sys/types.h>
#include <sys/ioctl.h>
#include <fcntl.h>
#include <linux/videodev.h>

char *video_device="/dev/video0";

int main(int argc, char **argv)
{
	int res;
	int fd;

	struct video_capability vcap;
	struct video_channel 	vchan;
	struct video_picture 	vpic;
	
	if (argc>1) {
		video_device=argv[1];
	}
	printf("Using device %s\n", video_device);

	fd = open(video_device, O_RDWR);
	if (fd<0) {
		perror("open");
		return -1;
	}

	res = ioctl(fd, VIDIOCGCAP, &vcap);
	if (res<0) {
		perror("VIDIOCGCAP");
	} else {
		printf("struct video_capability\n");
		printf("{\n");
		printf("	name='%s'\n", vcap.name);
		printf("	channels=%d\n", vcap.channels);
		printf("	audios=%d\n", vcap.audios);
		printf("	maxwidth=%d\n", vcap.maxwidth);
		printf("	maxheight=%d\n", vcap.maxheight);
		printf("	minwidth=%d\n", vcap.minwidth);
		printf("	minheight=%d\n", vcap.minheight);
		printf("}\n");
		
	}
	
	res = ioctl(fd, VIDIOCGCHAN, &vchan);
	if (res<0) {
		perror("VIDIOCGCHAN");
	}

	res = ioctl(fd, VIDIOCGPICT, &vpic);
	if (res<0) {
		perror("VIDIOCGPICT");
	} else {
		printf("struct video_picture\n");
		printf("{\n");
		printf("	brightness=%d\n", vpic.brightness);
		printf("	hue=%d\n", vpic.hue);
		printf("	colour=%d\n", vpic.colour);
		printf("	contrast=%d\n", vpic.contrast);
		printf("	whiteness=%d\n", vpic.whiteness);
		printf("	depth=%d\n", vpic.depth);
		printf("	palette=%d\n", vpic.palette);
		printf("}\n");
	}
	
	close(fd);
	
	return 0;
}

