/*
 * v4l_tools    A small set of tools for video4linux
 * Copyright (C) 2002, 2003 Raphael Assenat
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Library General Public
 * License as published by the Free Software Foundation; either
 * version 2 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warrenty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Library General Public License for more details.
 *
 * You should have received a copy of the GNU Library General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307 USA
 *
 * Raphael Assenat
 * raph@raphnet.net
 * http://www.raphnet.net
 *
 * */
#include <stdio.h>
#include <unistd.h>
#include <stdlib.h>
#include <string.h>
#include <sys/stat.h>
#include <sys/types.h>
#include <sys/ioctl.h>
#include <fcntl.h>
#include <linux/videodev.h>

// vars
char *video_device="/dev/video0";

// proto
int try_palette(int fd, unsigned short palette, struct video_picture *vpic);

int main(int argc, char **argv)
{
	int res;
	int fd=-1;

	struct video_picture 	vpic;
	struct video_picture 	ini_vpic;
	
	if (argc>1) {
		video_device=argv[1];
	}
	printf("Using device %s\n", video_device);

	fd = open(video_device, O_RDWR);
	if (fd<0) {
		perror("open");
		return -1;
	}

	/* Read the initial settings */
	res = ioctl(fd, VIDIOCGPICT, &ini_vpic);
	if (res<0) {
		perror("VIDIOCGPICT");
		goto shutdown;
	} 
	
	memcpy(&vpic, &ini_vpic, sizeof(struct video_picture));
	
	printf("Supported palettes:\n\n");
	
	printf("VIDEO_PALETTE_GREY: ");
	if (try_palette(fd, VIDEO_PALETTE_GREY, &vpic)) { printf("YES\n"); } else { printf("NO\n"); }
	printf("VIDEO_PALETTE_HI240: ");
	if (try_palette(fd, VIDEO_PALETTE_HI240, &vpic)) { printf("YES\n"); } else { printf("NO\n"); }
	printf("VIDEO_PALETTE_RGB565: ");
	if (try_palette(fd, VIDEO_PALETTE_RGB565, &vpic)) { printf("YES\n"); } else { printf("NO\n"); }
	printf("VIDEO_PALETTE_RGB24: ");
	if (try_palette(fd, VIDEO_PALETTE_RGB24, &vpic)) { printf("YES\n"); } else { printf("NO\n"); }
	printf("VIDEO_PALETTE_RGB32: ");
	if (try_palette(fd, VIDEO_PALETTE_RGB32, &vpic)) { printf("YES\n"); } else { printf("NO\n"); }
	printf("VIDEO_PALETTE_RGB555: ");
	if (try_palette(fd, VIDEO_PALETTE_RGB555, &vpic)) { printf("YES\n"); } else { printf("NO\n"); }
	printf("VIDEO_PALETTE_YUV422: ");
	if (try_palette(fd, VIDEO_PALETTE_YUV422, &vpic)) { printf("YES\n"); } else { printf("NO\n"); }
	printf("VIDEO_PALETTE_YUV411: ");
	if (try_palette(fd, VIDEO_PALETTE_YUV411, &vpic)) { printf("YES\n"); } else { printf("NO\n"); }
	printf("VIDEO_PALETTE_RAW: ");
	if (try_palette(fd, VIDEO_PALETTE_RAW, &vpic)) { printf("YES\n"); } else { printf("NO\n"); }
	printf("VIDEO_PALETTE_YUV422P: ");
	if (try_palette(fd, VIDEO_PALETTE_YUV422P, &vpic)) { printf("YES\n"); } else { printf("NO\n"); }
	printf("VIDEO_PALETTE_YUV411P: ");
	if (try_palette(fd, VIDEO_PALETTE_YUV411P, &vpic)) { printf("YES\n"); } else { printf("NO\n"); }
	
	printf("VIDEO_PALETTE_YUV420P: ");
	if (try_palette(fd, VIDEO_PALETTE_YUV420P, &vpic)) { printf("YES\n"); } else { printf("NO\n"); }
	printf("VIDEO_PALETTE_YUV410P: ");
	if (try_palette(fd, VIDEO_PALETTE_YUV410P, &vpic)) { printf("YES\n"); } else { printf("NO\n"); }
		
	
	if (ioctl(fd, VIDIOCSPICT, &ini_vpic)<0) { return 0; }
	
shutdown:
	if (fd!=-1) { close(fd); }
	
	return 0;
}

/* set the video picture using the given palette. Returns 1 if it seems
 * to work, otherwise -1 */
int try_palette(int fd, unsigned short palette, struct video_picture *vpic)
{
	struct video_picture gpic;
	
	vpic->palette = palette;
	
	if (ioctl(fd, VIDIOCSPICT, vpic)<0) { return 0; }

	ioctl(fd, VIDIOCGPICT, &gpic);
	if (gpic.palette!=vpic->palette) { printf("MISMATCH. "); return 0; }
	
	
	return 1;
}


