/*
 * v4l_tools    A small set of tools for video4linux
 * Copyright (C) 2002, 2003 Raphael Assenat
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Library General Public
 * License as published by the Free Software Foundation; either
 * version 2 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warrenty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Library General Public License for more details.
 *
 * You should have received a copy of the GNU Library General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307 USA
 *
 * Raphael Assenat
 * raph@raphnet.net
 * http://www.raphnet.net
 *
 * */
#include <stdio.h>
#include <unistd.h>
#include <stdlib.h>
#include <string.h>
#include <sys/stat.h>
#include <sys/types.h>
#include <sys/ioctl.h>
#include <fcntl.h>
#include <linux/videodev.h>

// vars
char *video_device="/dev/video0";
char *palette="";

// proto
int try_palette(int fd, unsigned short palette, struct video_picture *vpic);
int getPaletteId(char *palette);

int main(int argc, char **argv)
{
	int res;
	int fd=-1;

	struct video_picture 	vpic;
	
	if (argc<2) {
		printf("Usage: ./v4l_set_palette palette [device]\n");
		printf("VIDEO_PALETTE_GREY\n");
		printf("VIDEO_PALETTE_HI240,\n");
		printf("VIDEO_PALETTE_RGB565\n");
		printf("VIDEO_PALETTE_RGB24\n");
		printf("VIDEO_PALETTE_RGB32\n");
		printf("VIDEO_PALETTE_RGB555\n");
		printf("VIDEO_PALETTE_YUV422\n");
		printf("VIDEO_PALETTE_YUYV\n");
		printf("VIDEO_PALETTE_UYVY\n");
		printf("VIDEO_PALETTE_YUV420\n");
		printf("VIDEO_PALETTE_YUV411\n");
		printf("VIDEO_PALETTE_RAW\n");
		printf("VIDEO_PALETTE_YUV422P\n");
		printf("VIDEO_PALETTE_YUV411P\n");
		printf("VIDEO_PALETTE_YUV420P\n");
		printf("VIDEO_PALETTE_YUV410P\n");
		printf("VIDEO_PALETTE_PLANAR\n");
		printf("VIDEO_PALETTE_COMPONENT\n\n");
		return 0;	
	}
	
	if (argc>2) {
		video_device=argv[2];
	}

	palette = argv[1];
	
	printf("Using device %s\n", video_device);
	printf("Setting palette %s\n", palette);

	fd = open(video_device, O_RDWR);
	if (fd<0) {
		perror("open");
		return -1;
	}

	/* Read the initial settings */
	res = ioctl(fd, VIDIOCGPICT, &vpic);
	if (res<0) {
		perror("VIDIOCGPICT");
		goto shutdown;
	} 

	vpic.palette = getPaletteId(palette);
	if (vpic.palette==0) {
		fprintf(stderr, "Unknown palette\n");
		goto shutdown;
	}
	
	if (try_palette(fd, vpic.palette, &vpic)==0) {
		fprintf(stderr, "Could not set palette\n");
	}
	
shutdown:
	if (fd!=-1) { close(fd); }
	
	return 0;
}


int getPaletteId(char *palette)
{

	if (strcasecmp(palette, "VIDEO_PALETTE_GREY")==0) { return VIDEO_PALETTE_GREY; }
	if (strcasecmp(palette, "VIDEO_PALETTE_HI240")==0) { return VIDEO_PALETTE_HI240; }
	if (strcasecmp(palette, "VIDEO_PALETTE_RGB565")==0) { return VIDEO_PALETTE_RGB565; }
	if (strcasecmp(palette, "VIDEO_PALETTE_RGB24")==0) { return VIDEO_PALETTE_RGB24; }
	if (strcasecmp(palette, "VIDEO_PALETTE_RGB32")==0) { return VIDEO_PALETTE_RGB32; }
	if (strcasecmp(palette, "VIDEO_PALETTE_RGB555")==0) { return VIDEO_PALETTE_RGB555; }
	if (strcasecmp(palette, "VIDEO_PALETTE_YUV422")==0) { return VIDEO_PALETTE_YUV422; }
	if (strcasecmp(palette, "VIDEO_PALETTE_YUYV")==0) { return VIDEO_PALETTE_YUYV; }
	if (strcasecmp(palette, "VIDEO_PALETTE_UYVY")==0) { return VIDEO_PALETTE_UYVY; }
	if (strcasecmp(palette, "VIDEO_PALETTE_YUV420")==0) { return VIDEO_PALETTE_YUV420; }
	if (strcasecmp(palette, "VIDEO_PALETTE_YUV411")==0) { return VIDEO_PALETTE_YUV411; }
	if (strcasecmp(palette, "VIDEO_PALETTE_RAW")==0) { return VIDEO_PALETTE_RAW; }
	if (strcasecmp(palette, "VIDEO_PALETTE_YUV422P")==0) { return VIDEO_PALETTE_YUV422P; }
	if (strcasecmp(palette, "VIDEO_PALETTE_YUV411P")==0) { return VIDEO_PALETTE_YUV411P; }
	if (strcasecmp(palette, "VIDEO_PALETTE_YUV420P")==0) { return VIDEO_PALETTE_YUV420P; }
	if (strcasecmp(palette, "VIDEO_PALETTE_YUV410P")==0) { return VIDEO_PALETTE_YUV410P; }

	return 0;
}


/* set the video picture using the given palette. Returns 1 if it seems
 * to work, otherwise 0 */
int try_palette(int fd, unsigned short palette, struct video_picture *vpic)
{
	struct video_picture gpic;
	
	vpic->palette = palette;
	
	if (ioctl(fd, VIDIOCSPICT, vpic)<0) { return 0; }

	ioctl(fd, VIDIOCGPICT, &gpic);
	if (gpic.palette!=vpic->palette) { printf("MISMATCH. "); return 0; }
	
	
	return 1;
}


